from __future__ import annotations

import abc

import scrapbook
import scrapbook.encoders

# NOTE this is supposed to be in the upstream
class BaseEncoder(abc.ABC):
    def name(self):
        ...

    def encodable(self, data):
        ...

    def encode(self, scrap: scrapbook.scraps.Scrap, **kwargs):
        ...

    def decode(self, scrap: scrapbook.scraps.Scrap, **kwargs):
        ...

# NOTE this is supposed to be in the upstream
def load(encoder: BaseEncoder):
    return scrapbook.encoders.registry.register(encoder)

__all__ = [
    BaseEncoder,
    load
]



from __future__ import annotations

import base64
import pickle
import functools

import scrapbook

import scrapbook.encoders
import scrapbook.scraps

from . import base
from .. import utils


class PickleEncoder(base.BaseEncoder):
    ENCODER_NAME = 'pickle'

    def name(self):
        return self.ENCODER_NAME

    def encodable(self, data):
        # TODO
        return True

    def encode(self, scrap: scrapbook.scraps.Scrap, **kwargs):
        _impl = utils.func.pipeline(
            functools.partial(pickle.dumps, **kwargs),
            # NOTE .decode() makes sure its a UTF-8 string instead of bytes
            lambda x: base64.b64encode(x).decode()
        )
        return scrap._replace(
            data=_impl(scrap.data)
        )

    def decode(self, scrap: scrapbook.scraps.Scrap, **kwargs):
        _impl = utils.func.pipeline(
            base64.b64decode,
            functools.partial(pickle.loads, **kwargs)
        )
        return scrap._replace(data=_impl(scrap.data))

def load():
    return base.load(PickleEncoder())

__all__= [
    PickleEncoder,
    load
]
